@@ -1,5 +1,5 @@
-/* Copyright (C) 2000-2006 Boris Wesslowski */
-/* $Id: modes.c,v 1.83 2006/03/08 19:36:03 bw Exp $ */
+/* Copyright (C) 2000-2010 Boris Wesslowski */
+/* $Id: modes.c 706 2010-10-06 12:02:36Z bw $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,7 +25,6 @@
 #include "parser.h"
 #include "output.h"
 #include "compare.h"
-#include "report.h"
 #include "response.h"
 #include "utils.h"
 #include "net.h"
@@ -96,7 +95,7 @@ void mode_summary()
 {
   char nows[TIMESIZE], first_entry[TIMESIZE], last_entry[TIMESIZE], *input = NULL, last_file = 0;
   FILE *output = NULL;
-  int retval, linenum = 0, hitnum = 0, errnum = 0, oldnum = 0, exnum = 0;
+  int retval, linenum = 0, hitnum = 0, errnum = 0, old_errnum = 0, oldnum = 0, exnum = 0;
   time_t now;
   struct passwd *gen_user;
   struct input_file *file;
@@ -137,12 +136,13 @@ void mode_summary()
 
     if (opt.verbose == 2)
       fprintf(stderr, "\n");
-    if (errnum && opt.verbose) {
+    if (opt.verbose && (errnum > old_errnum)) {
       fprintf(stderr, _("Unrecognized entries or tokens can be submitted at\n"));
       fprintf(stderr, "http://fwlogwatch.inside-security.de/unrecognized.php\n");
+      old_errnum = errnum;
     }
 
-    if(opt.std_in) {
+    if (opt.std_in) {
       last_file++;
     } else {
       if (opt.verbose)
@@ -177,19 +177,30 @@ void mode_summary()
   if (opt.verbose)
     fprintf(stderr, _("Sorting data\n"));
 
-  if(first != NULL) {
+  if (first != NULL) {
+    time_t last_time;
+    struct conn_data *p;
+
     opt.sortfield = SORT_END_TIME;
     opt.sortmode = ORDER_DESCENDING;
     first = fwlw_pc_mergesort(first);
-    if(opt.verbose == 2)
+    if (opt.verbose == 2)
       fprintf(stderr, ".");
-    strftime(last_entry, TIMESIZE, _("%b %d %H:%M:%S"), localtime(&first->end_time));
+    last_time = first->end_time;
+
     opt.sortfield = SORT_START_TIME;
     opt.sortmode = ORDER_ASCENDING;
     first = fwlw_pc_mergesort(first);
-    if(opt.verbose == 2)
+    if (opt.verbose == 2)
       fprintf(stderr, ".");
     strftime(first_entry, TIMESIZE, _("%b %d %H:%M:%S"), localtime(&first->start_time));
+
+    p = first;
+    while (p->next != NULL)
+      p = p->next;
+    if (p->start_time > last_time)
+      last_time = p->start_time;
+    strftime(last_entry, TIMESIZE, _("%b %d %H:%M:%S"), localtime(&last_time));
   } else {
     first_entry[0] = '\0';
   }
@@ -208,15 +219,15 @@ void mode_summary()
       fprintf(stderr, "freopen %s: %s\n", opt.outputfile, strerror(errno));
       exit(EXIT_FAILURE);
     }
-  } else if ((opt.mode != INTERACTIVE_REPORT) && (opt.recipient[0] != '\0')) {
+  } else if (opt.recipient[0] != '\0') {
     char buf[BUFSIZE];
 
-    if(opt.verbose)
+    if (opt.verbose)
       fprintf(stderr, _("Sending\n"));
 
     snprintf(buf, BUFSIZE, "%s -t", P_SENDMAIL);
     output = popen(buf, "w");
-    if(output == NULL) {
+    if (output == NULL) {
       perror("popen");
       exit(EXIT_FAILURE);
     }
@@ -267,7 +278,7 @@ void mode_summary()
     fprintf(output, _("of %d entries in %d input files are packet logs, "), linenum, opt.filecount);
   }
   retval = list_stats();
-  if(retval == 1) {
+  if (retval == 1) {
     fprintf(output, _("one has unique characteristics.\n"));
   } else {
     fprintf(output, _("%d have unique characteristics.\n"), retval);
@@ -277,7 +288,7 @@ void mode_summary()
     if (opt.html)
       fprintf(output, "<br />\n");
 
-    if(exnum == 1) {
+    if (exnum == 1) {
       fprintf(output, _("One entry was excluded by configuration.\n"));
     } else {
       fprintf(output, _("%d entries were excluded by configuration.\n"), exnum);
@@ -293,43 +304,43 @@ void mode_summary()
     fprintf(output, _("No valid time entries found.\n"));
   }
 
-  if(!opt.loghost) {
-    if(opt.html)
+  if (!opt.loghost) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("All entries were logged by the same host: \"%s\".\n"), opt.hostname);
   }
 
-  if(!opt.chains) {
-    if(opt.html)
+  if (!opt.chains) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("All entries are from the same chain: \"%s\".\n"), opt.chainlabel);
   }
 
-  if(!opt.branches) {
-    if(opt.html)
+  if (!opt.branches) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("All entries have the same target: \"%s\".\n"), opt.branchname);
   }
 
-  if(!opt.ifs) {
-    if(opt.html)
+  if (!opt.ifs) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("All entries are from the same interface: \"%s\".\n"), opt.interface);
   }
 
-  if(opt.least > 1) {
-    if(opt.html)
+  if (opt.least > 1) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("Only entries with a count of at least %d are shown.\n"), opt.least);
   }
 
-  if(opt.max) {
-    if(opt.html)
+  if (opt.max) {
+    if (opt.html)
       fprintf(output, "<br />\n");
 
     fprintf(output, _("Only the top %d entries are shown.\n"), opt.max);
@@ -340,16 +351,13 @@ void mode_summary()
   else
     fprintf(output, "\n");
 
-  if(opt.mode == INTERACTIVE_REPORT)
-    fprintf(output, _("Reporting threshold: %d\n\n"), opt.threshold);
-
 #ifdef HAVE_ADNS
-  if(opt.resolve) {
-    if(opt.verbose)
+  if (opt.resolve) {
+    if (opt.verbose)
       fprintf(stderr, _("Resolving\n"));
 
     retval = adns_init(&adns, adns_if_noenv, 0);
-    if(retval) {
+    if (retval) {
       perror("adns_init");
       exit(EXIT_FAILURE);
     }
@@ -357,20 +365,17 @@ void mode_summary()
   }
 #endif
 
-  if(opt.whois_lookup)
+  if (opt.whois_lookup)
     whois_connect(RADB);
 
   show_list(output);
   fflush(output);
 
-  if(opt.whois_lookup)
+  if (opt.whois_lookup)
     whois_close();
 
-  if(opt.mode == INTERACTIVE_REPORT)
-    report();
-
 #ifdef HAVE_ADNS
-  if(opt.resolve)
+  if (opt.resolve)
     adns_finish(adns);
 #endif
 
@@ -394,7 +399,7 @@ void mode_summary()
     if (retval == EOF) {
       perror("fclose");
     }
-  } else if ((opt.mode != INTERACTIVE_REPORT) && (opt.recipient[0] != '\0')) {
+  } else if (opt.recipient[0] != '\0') {
     retval = pclose(output);
     if (retval == -1) {
       perror("pclose");
@@ -410,7 +415,7 @@ void check_pidfile()
   if (stat(opt.pidfile, sbuf) != -1) {
     fprintf(stderr, _("Warning: pidfile exists, another fwlogwatch might be running.\n"));
   } else {
-    if ((errno != ENOENT) && (errno != EACCES)){
+    if ((errno != ENOENT) && (errno != EACCES)) {
       fprintf(stderr, "stat %s: %d, %s\n", opt.pidfile, errno, strerror(errno));
       exit(EXIT_FAILURE);
     }
@@ -421,7 +426,7 @@ void check_pidfile()
 void mode_rt_response_reread_conf()
 {
   free_exclude_data();
-  if(read_rcfile(opt.rcfile, MAY_NOT_EXIST) == EXIT_SUCCESS) {
+  if (read_rcfile(opt.rcfile, MAY_NOT_EXIST) == EXIT_SUCCESS) {
     syslog(LOG_NOTICE, _("SIGHUP caught, reread configuration file %s"), opt.rcfile);
   } else {
     syslog(LOG_NOTICE, _("SIGHUP caught, unable to reread configuration file %s"), opt.rcfile);
@@ -431,7 +436,7 @@ void mode_rt_response_reread_conf()
 
 void mode_rt_response_open()
 {
-  if(opt.std_in) {
+  if (opt.std_in) {
     opt.inputfd = stdin;
   } else {
     opt.inputfd = fopen(first_file->name, "r");
@@ -446,13 +451,13 @@ void mode_rt_response_reopen_log()
 {
   int retval;
 
-  if(opt.std_in) {
+  if (opt.std_in) {
     syslog(LOG_NOTICE, _("SIGUSR1 caught, reading input from stdin, no need to reopen log file"));
   } else {
     syslog(LOG_NOTICE, _("SIGUSR1 caught, reopening log file %s"), first_file->name);
 
     retval = fclose(opt.inputfd);
-    if(retval == EOF)
+    if (retval == EOF)
       syslog(LOG_NOTICE, "fclose %s: %s", first_file->name, strerror(errno));
 
     mode_rt_response_open();
@@ -468,7 +473,7 @@ void mode_rt_response_core()
   fd_set rfds;
   struct timeval tv;
 
-  if((!opt.std_in) && (!opt.stateful_start)) {
+  if ((!opt.std_in) && (!opt.stateful_start)) {
     retval = fstat(fileno(opt.inputfd), &info);
     if (retval == -1) {
       syslog(LOG_NOTICE, "fstat %s: %s", first_file->name, strerror(errno));
@@ -480,14 +485,14 @@ void mode_rt_response_core()
   opt.line = xmalloc(sizeof(struct log_line));
 
   while (1) {
-    if(opt.status) {
+    if (opt.status) {
       FD_ZERO(&rfds);
       FD_SET(opt.sock, &rfds);
       tv.tv_sec = 1;
       tv.tv_usec = 0;
-      retval = select(opt.sock+1, &rfds, NULL, NULL, &tv);
+      retval = select(opt.sock + 1, &rfds, NULL, NULL, &tv);
       if (retval == -1) {
-	if(errno != EINTR) {
+	if (errno != EINTR) {
 	  syslog(LOG_NOTICE, "select: %s", strerror(errno));
 	  exit(EXIT_FAILURE);
 	}
@@ -499,8 +504,8 @@ void mode_rt_response_core()
       sleep(1);
     }
 
-    remove_old(RESP_REMOVE_OPC|RESP_REMOVE_OHS);
-    if(opt.std_in) {
+    remove_old(RESP_REMOVE_OPC | RESP_REMOVE_OHS);
+    if (opt.std_in) {
       common_input_loop(&linenum, &hitnum, &ignored, &ignored, &ignored);
       look_for_alert();
     } else {
@@ -509,7 +514,7 @@ void mode_rt_response_core()
 	syslog(LOG_NOTICE, "fstat %s: %s", first_file->name, strerror(errno));
 	log_exit(EXIT_FAILURE);
       }
-      if(size != info.st_size) {
+      if (size != info.st_size) {
 	size = info.st_size;
 	clearerr(opt.inputfd);
 	common_input_loop(&linenum, &hitnum, &ignored, &ignored, &ignored);
@@ -523,7 +528,7 @@ void mode_rt_response_terminate()
 {
   syslog(LOG_NOTICE, _("SIGTERM caught, cleaning up"));
   free_hosts();
-  if(opt.response & OPT_RESPOND)
+  if (opt.response & OPT_RESPOND)
     modify_firewall(FW_STOP);
   log_exit(EXIT_SUCCESS);
 }
@@ -535,7 +540,7 @@ void mode_rt_response()
 #ifndef RR_DEBUG
   pid_t pid;
 
-  if(opt.pidfile[0] != '\0')
+  if (opt.pidfile[0] != '\0')
     check_pidfile();
 
   pid = fork();
@@ -575,14 +580,14 @@ void mode_rt_response()
     perror("close");
     exit(EXIT_FAILURE);
   }
-  if(!opt.std_in) {
+  if (!opt.std_in) {
     retval = close(0);
     if (retval == -1) {
       perror("close");
       exit(EXIT_FAILURE);
     }
   }
-  retval = open("/dev/null",O_RDWR);
+  retval = open("/dev/null", O_RDWR);
   if (retval == -1) {
     perror("open");
     exit(EXIT_FAILURE);
@@ -592,7 +597,7 @@ void mode_rt_response()
     perror("dup");
     exit(EXIT_FAILURE);
   }
-  if(!opt.std_in) {
+  if (!opt.std_in) {
     retval = dup(0);
     if (retval == -1) {
       perror("dup");
@@ -601,18 +606,18 @@ void mode_rt_response()
   }
   openlog("fwlogwatch", LOG_CONS, LOG_DAEMON);
 #else
-  openlog("fwlogwatch", LOG_CONS|LOG_PERROR, LOG_DAEMON);
+  openlog("fwlogwatch", LOG_CONS | LOG_PERROR, LOG_DAEMON);
 #endif
   syslog(LOG_NOTICE, _("Starting (pid %d)"), getpid());
 
   signal(SIGTERM, mode_rt_response_terminate);
 
-  if(opt.pidfile[0] != '\0') {
+  if (opt.pidfile[0] != '\0') {
     pidfile = fopen(opt.pidfile, "w");
     if (pidfile == NULL) {
       syslog(LOG_NOTICE, "fopen %s: %s\n", opt.pidfile, strerror(errno));
     } else {
-      fprintf(pidfile, "%d\n", (int)getpid());
+      fprintf(pidfile, "%d\n", (int) getpid());
       retval = fclose(pidfile);
       if (retval == EOF) {
 	syslog(LOG_NOTICE, "fclose %s: %s\n", opt.pidfile, strerror(errno));
@@ -620,12 +625,12 @@ void mode_rt_response()
     }
   }
 
-  if(opt.status) {
+  if (opt.status) {
     prepare_socket();
 #ifdef HAVE_ADNS
-    if(opt.resolve) {
+    if (opt.resolve) {
       retval = adns_init(&adns, adns_if_noenv, 0);
-      if(retval) {
+      if (retval) {
 	syslog(LOG_NOTICE, "adns_init: %s", strerror(errno));
 	log_exit(EXIT_FAILURE);
       }
@@ -633,26 +638,26 @@ void mode_rt_response()
 #endif
   }
 
-  if((opt.ipchains_check == 1) && ((opt.format & PARSER_IPCHAINS) != 0))
+  if ((opt.ipchains_check == 1) && ((opt.format & PARSER_IPCHAINS) != 0))
     check_for_ipchains();
 
-  if((opt.response & OPT_NOTIFY) != 0)
+  if ((opt.response & OPT_NOTIFY) != 0)
     check_script_perms(opt.notify_script);
 
-  if((opt.response & OPT_RESPOND) != 0) {
+  if ((opt.response & OPT_RESPOND) != 0) {
     check_script_perms(opt.respond_script);
     modify_firewall(FW_START);
   }
 
   mode_rt_response_open();
 
-  if(opt.run_as[0] != '\0') {
+  if (opt.run_as[0] != '\0') {
     uid_t olduid;
     gid_t oldgid;
     struct passwd *pwe;
 
     pwe = getpwnam(opt.run_as);
-    if(pwe == NULL) {
+    if (pwe == NULL) {
       syslog(LOG_NOTICE, _("User to run as was not found"));
       log_exit(EXIT_FAILURE);
     }
@@ -673,25 +678,21 @@ void mode_rt_response()
     syslog(LOG_NOTICE, _("Running with uid %d, gid %d"), getuid(), getgid());
   }
 
-  if(opt.threshold == 1) {
+  if (opt.threshold == 1) {
     syslog(LOG_NOTICE, _("Alert threshold is one attempt"));
   } else {
     syslog(LOG_NOTICE, _("Alert threshold is %d attempts"), opt.threshold);
   }
 
-  if(opt.recent < 3600) {
-    syslog(LOG_NOTICE, _("Events older than %d second(s) are discarded"),
-	   opt.recent);
+  if (opt.recent < 3600) {
+    syslog(LOG_NOTICE, _("Events older than %d second(s) are discarded"), opt.recent);
   } else {
-    syslog(LOG_NOTICE, _("Events older than %d hour(s) are discarded"),
-	   opt.recent/3600);
+    syslog(LOG_NOTICE, _("Events older than %d hour(s) are discarded"), opt.recent / 3600);
   }
 
-  syslog(LOG_NOTICE, _("Response mode: Log%s%s"),
-	 (opt.response & OPT_NOTIFY)?_(", notify"):"",
-	 (opt.response & OPT_RESPOND)?_(", respond"):"");
+  syslog(LOG_NOTICE, _("Response mode: Log%s%s"), (opt.response & OPT_NOTIFY) ? _(", notify") : "", (opt.response & OPT_RESPOND) ? _(", respond") : "");
 
-  if((!opt.std_in) && (!opt.stateful_start)) {
+  if ((!opt.std_in) && (!opt.stateful_start)) {
     retval = fseek(opt.inputfd, 0, SEEK_END);
     if (retval == -1) {
       syslog(LOG_NOTICE, "fseek %s: %s", first_file->name, strerror(errno));
@@ -706,8 +707,9 @@ void mode_rt_response()
 
 void mode_show_log_times()
 {
-  char buf[BUFSIZE], stime[TIMESIZE], month[3], *input = NULL, last_file = 0;
-  int retval = 0, loop, day, hour, minute, second, linenum = 0;
+  char buf[BUFSIZE], stime[TIMESIZE], month[4], *input = NULL, last_file = 0;
+  int retval = 0, loop, hour, minute, second, linenum = 0;
+  unsigned int day;
   struct input_file *file;
   time_t first = 0, last = 0;
 
@@ -752,8 +754,8 @@ void mode_show_log_times()
 
     while (loop) {
       linenum++;
-      retval = sscanf(buf, "%3s %2d %2d:%2d:%2d ", month, &day, &hour, &minute, &second);
-      if(retval == 5) {
+      retval = sscanf(buf, "%3s %u %2d:%2d:%2d ", month, &day, &hour, &minute, &second);
+      if (retval == 5) {
 	build_time(month, day, hour, minute, second);
 	if (first == 0)
 	  first = last = opt.line->time;
@@ -762,7 +764,6 @@ void mode_show_log_times()
 	if (opt.line->time > last)
 	  last = opt.line->time;
       }
-
 #ifdef HAVE_ZLIB
       if (opt.std_in) {
 #endif
@@ -774,7 +775,7 @@ void mode_show_log_times()
 #endif
     }
 
-    if(opt.std_in) {
+    if (opt.std_in) {
       last_file++;
     } else {
       if (opt.verbose)
